@@ -1,98 +1,45 @@
-# -*- coding: utf-8 -*-
-
-from pickle import dumps, loads
-from base64 import b64encode, b64decode
-from time import time
-from smtplib import SMTP
-
-from Crypto.Cipher import Blowfish
-
 from logilab.common.decorators import clear_cache
 
-from cubicweb.common import tags, mail
+from cubicweb import mail, crypto
 from cubicweb.view import StartupView
 from cubicweb.web import Redirect, ValidationError
-from cubicweb.web import (controller, httpcache, form,
-                          formwidgets as wdg, formfields as ff)
+from cubicweb.web import controller, form, captcha
+from cubicweb.web import formwidgets as fw, formfields as ff
 from cubicweb.web.views import forms, basecomponents, urlrewrite
 
-from cubes.registration import captcha
-
-
-_CYPHERERS = {}
-def _cypherer(seed):
-    try:
-        return _CYPHERERS[seed]
-    except KeyError:
-        _CYPHERERS[seed] = Blowfish.new(seed, Blowfish.MODE_ECB)
-        return _CYPHERERS[seed]
-
-
-def encrypt(data, seed):
-    string = dumps(data)
-    string = string + '*' * (8 - len(string) % 8)
-    string = b64encode(_cypherer(seed).encrypt(string))
-    return unicode(string)
-
-
-def decrypt(string, seed):
-    # pickle ignores trailing characters so we do not need to strip them off
-    string = _cypherer(seed).decrypt(b64decode(string))
-    return loads(string)
-
-
-class CaptchaView(StartupView):
-    http_cache_manager = httpcache.NoHTTPCacheManager
-    id = 'captcha'
-    binary = True
-    templatable = False
-    content_type = 'image/jpg'
-
-    def call(self):
-        text, data = captcha.captcha(self.config['captcha-font-file'],
-                                     self.config['captcha-font-size'])
-        self.req.set_session_data('captcha', text)
-        self.w(data.read())
-
-
-class CaptchaWidget(wdg.TextInput):
-    def render(self, form, field):
-        name, curvalues, attrs = self._render_attrs(form, field)
-        # t=int(time()*100) to make sure img is not cached
-        src = form.req.build_url('view', vid='captcha', t=int(time()*100))
-        img = tags.img(src=src, alt=u'captcha')
-        img = u'<div class="captcha">%s</div>' % img
-        return img + super(CaptchaWidget, self).render(form, field)
 
 
 class RegistrationForm(forms.FieldsForm):
-    id = 'registration'
-    form_buttons = [wdg.SubmitButton()]
+    __regid__ = 'registration'
+    domid = 'registrationForm'
+    form_buttons = [fw.SubmitButton()]
 
     @property
     def action(self):
-        return self.req.build_url(u'registration_sendmail')
+        return self._cw.build_url(u'registration_sendmail')
 
-    login = ff.StringField(widget=wdg.TextInput(), required=True)
-    upassword = ff.StringField(widget=wdg.PasswordInput(), required=True)
-    use_email = ff.StringField(widget=wdg.TextInput(), required=True)
-    firstname = ff.StringField(widget=wdg.TextInput())
-    surname = ff.StringField(widget=wdg.TextInput())
-    captcha = ff.StringField(widget=CaptchaWidget(), required=True,
+    login = ff.StringField(widget=fw.TextInput(), required=True)
+    upassword = ff.StringField(widget=fw.PasswordInput(), required=True)
+    use_email = ff.StringField(widget=fw.TextInput(), required=True)
+    firstname = ff.StringField(widget=fw.TextInput())
+    surname = ff.StringField(widget=fw.TextInput())
+    captcha = ff.StringField(widget=captcha.CaptchaWidget(), required=True,
                              label=_('captcha'),
                              help=_('please copy the letters from the image'))
 
 
+# XXX move captcha validation to a CaptchaField
+
 class RegistrationFormView(form.FormViewMixIn, StartupView):
-    id = 'registration'
+    __regid__ = 'registration'
 
     def call(self):
-        form = self.vreg.select_object('forms', 'registration', self.req)
-        self.w(form.form_render())
+        form = self._cw.vreg['forms'].select('registration', self._cw)
+        self.w(form.render(display_progress_div=False))
 
 
 class RegistrationSendMailController(controller.Controller):
-    id = 'registration_sendmail'
+    __regid__ = 'registration_sendmail'
     content = _(u'''
 Hello %(firstname)s %(surname)s,
 
@@ -109,27 +56,29 @@ See you soon on %(base_url)s !
         data = self.checked_data()
         recipient = data['use_email']
         msg = self.build_email(recipient, data)
-        self.config.sendmails([(msg, (recipient,))])
+        self._cw.vreg.config.sendmails([(msg, (recipient,))])
         raise Redirect(self.success_redirect_url())
 
     def checked_data(self):
         '''only basic data check here (required attributes and password
         confirmation check)
         '''
-        fieldsform = self.vreg.select_object('forms', 'registration', self.req)
+        fieldsform = self._cw.vreg['forms'].select('registration', self._cw)
         data = {}
         errors = {}
         for field in fieldsform._fields_:
-            value = self.req.form.get(field.name, u'').strip()
+            value = self._cw.form.get(field.name, u'').strip()
             if not value:
                 if field.required:
-                    errors[field.name] = self.req._('required attribute')
+                    errors[field.name] = self._cw._('required attribute')
             data[field.name] = value
-        if data['upassword'] != self.req.form.get('upassword-confirm'):
+        if data['upassword'] != self._cw.form.get('upassword-confirm'):
             errors['upassword'] = _('passwords are different')
-        captcha = self.req.get_session_data('captcha', None, pop=True)
-        if data['captcha'].lower() != captcha.lower():
-            errors['captcha'] = self.req._('incorrect captcha value')
+        captcha = self._cw.get_session_data('captcha', None, pop=True)
+        if captcha is None:
+            errors[None] = self._cw._('unable to check captcha, please try again')
+        elif data['captcha'].lower() != captcha.lower():
+            errors['captcha'] = self._cw._('incorrect captcha value')
         if errors:
             raise ValidationError(None, errors)
         return data
@@ -137,30 +86,31 @@ See you soon on %(base_url)s !
     def build_email(self, recipient, data):
         if not data.get('firstname') or data.get('surtname'):
             data['firstname'] = data['login']
-        data.update({'base_url': self.config['base-url'],
+        data.update({'base_url': self._cw.vreg.config['base-url'],
                     'url': self.activation_url(data)})
-        content = self.req._(self.content) % data
-        subject = self.req._(self.subject) % data
+        content = self._cw._(self.content) % data
+        subject = self._cw._(self.subject) % data
         return mail.format_mail({}, [recipient], content=content,
-                                subject=subject, config=self.config)
+                                subject=subject, config=self._cw.vreg.config)
 
     def activation_url(self, data):
-        key = encrypt(data, self.config['cypher-seed'])
-        return self.build_url('registration_confirm', key=key)
+        key = crypto.encrypt(data, self._cw.vreg.config['registration-cypher-seed'])
+        return self._cw.build_url('registration_confirm', key=key)
 
     def success_redirect_url(self):
-        msg = self.req._(u'Your registration email has been sent. Follow '
+        msg = self._cw._(u'Your registration email has been sent. Follow '
                          'instructions in there to activate your account.')
-        return self.build_url('', __message=msg)
+        return self._cw.build_url('', __message=msg)
 
 
 class RegistrationConfirmController(controller.Controller):
-    id = 'registration_confirm'
+    __regid__ = 'registration_confirm'
 
     def publish(self, rset=None):
-        req = self.req
+        req = self._cw
         try:
-            data = decrypt(req.form['key'], self.vreg.config['cypher-seed'])
+            data = crypto.decrypt(req.form['key'],
+                                  req.vreg.config['registration-cypher-seed'])
             self.debug('registration data: %s', data)
         except:
             msg = req._(u'Invalid registration data. Please try registering again.')
@@ -191,19 +141,19 @@ class RegistrationConfirmController(controller.Controller):
         data.pop('upassword-confirm', None)
         data['__message'] = u'<br/>'.join(u'%s : %s' % (k,v)
                                           for (k,v) in errors.iteritems())
-        return self.build_url(u'register', **data)
+        return self._cw.build_url(u'register', **data)
 
     def success_redirect_url(self, data):
-        msg = self.req._(u'Congratulations, your registration is complete. '
+        msg = self._cw._(u'Congratulations, your registration is complete. '
                          'Welcome %(firstname)s %(surname)s !')
-        return self.build_url('', __message=msg%data)
+        return self._cw.build_url('', __message=msg%data)
 
 
 class UserLink(basecomponents.UserLink):
     def anon_user_link(self):
         super(UserLink, self).anon_user_link()
         self.w(u'&nbsp;[<a class="logout" href="%s">%s</a>]' % (
-            self.build_url('register'), self.req._('i18n_register_user')))
+            self._cw.build_url('register'), self._cw._('i18n_register_user')))
 
 ## urls #######################################################################
 
